'use strict';

var path = require('path');
var crypto = require('crypto');

const HASH_LENGTH = 7;
/**
 * Get content byte size
 */
function getByteSize(content) {
    if (typeof content === 'string') {
        return Buffer.from(content).length;
    }
    return content?.length || 0;
}
/**
 * Generate a 7 chars hash from a filepath
 */
function getHash(filepath) {
    const digest = crypto.createHash('sha256');
    return digest.update(Buffer.from(filepath)).digest('hex').substr(0, HASH_LENGTH);
}
function getChunkId(chunk) {
    let value = chunk.name;
    // Use entry module relative path
    if (chunk.moduleIds?.length > 0) {
        const absoluteModulePath = chunk.moduleIds[chunk.moduleIds.length - 1];
        value = path.relative(process.cwd(), absoluteModulePath);
    }
    return getHash([chunk, value].join('-'));
}
/**
 * Check if filepath should be excluded based on a config
 */
function checkExcludeFilepath(filepath, option) {
    if (!option) {
        return false;
    }
    if (Array.isArray(option)) {
        let res = false;
        for (let i = 0; i <= option.length - 1 && res === false; i++) {
            res = checkExcludeFilepath(filepath, option[i]);
        }
        return res;
    }
    if (typeof option === 'function') {
        return option(filepath);
    }
    if (typeof option === 'string') {
        return Boolean(filepath.match(option));
    }
    if ('test' in option) {
        return option.test(filepath);
    }
    return false;
}

const bundleToWebpackStats = (bundle, pluginOptions) => {
    const options = {
        moduleOriginalSize: false,
        ...pluginOptions,
    };
    const items = Object.values(bundle);
    const assets = [];
    const chunks = [];
    const moduleByFileName = {};
    items.forEach(item => {
        if (item.type === 'chunk') {
            if (checkExcludeFilepath(item.fileName, options.excludeAssets)) {
                return;
            }
            assets.push({
                name: item.fileName,
                size: getByteSize(item.code),
            });
            const chunkId = getChunkId(item);
            chunks.push({
                id: chunkId,
                entry: item.isEntry,
                initial: !item.isDynamicEntry,
                files: [item.fileName],
                names: [item.name],
            });
            Object.entries(item.modules).forEach(([modulePath, moduleInfo]) => {
                if (checkExcludeFilepath(modulePath, options.excludeModules)) {
                    return;
                }
                // Remove unexpected rollup null prefix
                const normalizedModulePath = modulePath.replace('\u0000', '');
                const relativeModulePath = path.relative(process.cwd(), normalizedModulePath);
                // Match webpack output - add current directory prefix for child modules
                const relativeModulePathWithPrefix = relativeModulePath.match(/^\.\./)
                    ? relativeModulePath
                    : `.${path.sep}${relativeModulePath}`;
                const moduleEntry = moduleByFileName[relativeModulePathWithPrefix];
                if (moduleEntry) {
                    moduleEntry.chunks.push(chunkId);
                }
                else {
                    moduleByFileName[relativeModulePathWithPrefix] = {
                        name: relativeModulePathWithPrefix,
                        size: options.moduleOriginalSize
                            ? moduleInfo.originalLength
                            : moduleInfo.renderedLength,
                        chunks: [chunkId],
                    };
                }
            });
        }
        else if (item.type === 'asset') {
            if (checkExcludeFilepath(item.fileName, options.excludeAssets)) {
                return;
            }
            assets.push({
                name: item.fileName,
                size: getByteSize(item.source.toString()),
            });
        }
        else ;
    });
    return {
        builtAt: Date.now(),
        assets,
        chunks,
        modules: Object.values(moduleByFileName),
    };
};

exports.bundleToWebpackStats = bundleToWebpackStats;
//# sourceMappingURL=transform.cjs.js.map
